@@ -249,7 +249,7 @@ LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL LIBXSMM_LOCK_TYPE internal_regl
   static LIBXSMM_TLS union { libxsmm_gemm_descriptor desc; char padding[LIBXSMM_GEMM_DESCRIPTOR_SIMD_SIZE]; } CACHE_KEYS[LIBXSMM_CACHESIZE]; \
   static LIBXSMM_TLS libxsmm_xmmfunction CACHE[LIBXSMM_CACHESIZE]; \
   static LIBXSMM_TLS unsigned int CACHE_ID = (unsigned int)(-1); \
-  static LIBXSMM_TLS unsigned int CACHE_HIT = LIBXSMM_CACHESIZE
+  static LIBXSMM_TLS unsigned int CACHE_HIT = LIBXSMM_CACHESIZE;
 # define INTERNAL_FIND_CODE_CACHE_BEGIN(CACHE_ID, CACHE_KEYS, CACHE, CACHE_HIT, RESULT, DESCRIPTOR) \
   assert(LIBXSMM_GEMM_DESCRIPTOR_SIMD_SIZE >= LIBXSMM_GEMM_DESCRIPTOR_SIZE); \
   /* search small cache starting with the last hit on record */ \
@@ -343,7 +343,7 @@ LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL LIBXSMM_LOCK_TYPE internal_regl
 #define INTERNAL_FIND_CODE(DESCRIPTOR, CODE) \
   internal_regentry flux_entry; \
 { \
-  INTERNAL_FIND_CODE_CACHE_DECL(cache_id, cache_keys, cache, cache_hit); \
+  INTERNAL_FIND_CODE_CACHE_DECL(cache_id, cache_keys, cache, cache_hit) \
   unsigned int hash, diff = 0, diff0 = 0, i0; \
   INTERNAL_FIND_CODE_INIT(CODE); \
   INTERNAL_FIND_CODE_CACHE_BEGIN(cache_id, cache_keys, cache, cache_hit, flux_entry, DESCRIPTOR) { \
@@ -461,9 +461,21 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_update_statistic(const libxsmm
       bucket = 1;
     }
 
-    /* TODO: use atomic updates */
-    internal_statistic[precision][bucket].ncol += ncol;
+#if (defined(_REENTRANT) || defined(LIBXSMM_OPENMP)) && defined(LIBXSMM_GCCATOMICS)
+# if (0 != LIBXSMM_GCCATOMICS)
+    /*dst =*/ __atomic_add_fetch(&internal_statistic[precision][bucket].ntry, ntry, __ATOMIC_RELAXED);
+    /*dst =*/ __atomic_add_fetch(&internal_statistic[precision][bucket].ncol, ncol, __ATOMIC_RELAXED);
+# else
+    /*dst =*/ __sync_add_and_fetch(&internal_statistic[precision][bucket].ntry, ntry);
+    /*dst =*/ __sync_add_and_fetch(&internal_statistic[precision][bucket].ncol, ncol);
+# endif
+#elif (defined(_REENTRANT) || defined(LIBXSMM_OPENMP)) && defined(_WIN32) /*TODO*/
     internal_statistic[precision][bucket].ntry += ntry;
+    internal_statistic[precision][bucket].ncol += ncol;
+#else
+    internal_statistic[precision][bucket].ntry += ntry;
+    internal_statistic[precision][bucket].ncol += ncol;
+#endif
   }
 }
 
@@ -531,9 +543,9 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE unsigned int internal_print_statistic(FILE*
   {
     char title[256], sml[256], med[256], big[256];
 
-    LIBXSMM_SNPRINTF(sml, sizeof(sml), "%u..%u",                     0u, internal_statistic_sml);
-    LIBXSMM_SNPRINTF(med, sizeof(sml), "%u..%u", internal_statistic_sml, internal_statistic_med);
-    LIBXSMM_SNPRINTF(big, sizeof(sml), "%u..%u", internal_statistic_med, internal_statistic_mnk);
+    LIBXSMM_SNPRINTF(sml, sizeof(sml), "%u..%u",                          0u, internal_statistic_sml);
+    LIBXSMM_SNPRINTF(med, sizeof(sml), "%u..%u", internal_statistic_sml + 1u, internal_statistic_med);
+    LIBXSMM_SNPRINTF(big, sizeof(sml), "%u..%u", internal_statistic_med + 1u, internal_statistic_mnk);
     {
       unsigned int n = 0;
       for (n = 0; 0 != target_arch[n] && n < sizeof(title); ++n) { /* toupper */
