@@ -111,7 +111,7 @@
      &        LIBXSMM_PREFETCH_BL2_VIA_C, LIBXSMM_PREFETCH_AL2_AHEAD)
 
         ! Enumerates the available target architectures and instruction
-        ! set extensions as returned by libxsmm_get_target_arch().
+        ! set extensions as returned by libxsmm_get_target_archid().
         INTEGER(C_INT), PARAMETER ::                                    &
      &    LIBXSMM_TARGET_ARCH_UNKNOWN = 0,                              &
      &    LIBXSMM_TARGET_ARCH_GENERIC = 1,                              &
@@ -122,7 +122,8 @@
      &    LIBXSMM_X86_SSE4_2       = 1004,                              &
      &    LIBXSMM_X86_AVX          = 1005,                              &
      &    LIBXSMM_X86_AVX2         = 1006,                              &
-     &    LIBXSMM_X86_AVX512       = 1007
+     &    LIBXSMM_X86_AVX512_MIC   = 1007,                              &
+     &    LIBXSMM_X86_AVX512_CORE  = 1008
 
         ! Type of a function specialized for a given parameter set.
         ABSTRACT INTERFACE
@@ -235,8 +236,15 @@
           MODULE PROCEDURE libxsmm_blas_sgemm, libxsmm_blas_dgemm
         END INTERFACE
 
+        ! Overloaded MATMUL-style routines (single/double precision).
+        INTERFACE libxsmm_matmul
+          MODULE PROCEDURE libxsmm_smatmul, libxsmm_dmatmul
+        END INTERFACE
+
         !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_init, libxsmm_finalize
-        !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_get_target_arch
+        !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_get_target_archid
+        !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_set_target_archid
+        !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_set_target_arch
         !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_timer_duration
         !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_timer_tick
         !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_omp_sgemm
@@ -250,14 +258,28 @@
           SUBROUTINE libxsmm_finalize() BIND(C)
           END SUBROUTINE
 
-          ! Returns the architecture and instruction set extension
-          ! as determined by the CPUID flags. 0 != LIBXSMM_JIT and
-          ! LIBXSMM_X86_AVX <= result, then this instruction set
-          ! extension is targeted by the JIT code generator.
-          INTEGER(C_INT) PURE FUNCTION libxsmm_get_target_arch() BIND(C)
+          ! Returns the architecture and instruction set extension as determined
+          ! by the CPUID flags, as set by the libxsmm_get_target_arch* functions,
+          ! or as set by the LIBXSMM_TARGET environment variable.
+          PURE FUNCTION libxsmm_get_target_archid() RESULT(id) BIND(C)
             IMPORT :: C_INT
+            INTEGER(C_INT) :: id
           END FUNCTION
 
+          ! Set target architecture (id: see PARAMETER enumeration)
+          ! for subsequent code generation (JIT).
+          SUBROUTINE libxsmm_set_target_archid(id) BIND(C)
+            IMPORT :: C_INT
+            INTEGER(C_INT), INTENT(IN), VALUE :: id
+          END SUBROUTINE
+
+          ! Set target architecture (arch="0|sse|snb|hsw|knl|skx", "0": CPUID)
+          ! for subsequent code generation (JIT).
+          SUBROUTINE libxsmm_set_target_arch(arch) BIND(C)
+            IMPORT :: C_CHAR
+            CHARACTER(C_CHAR), INTENT(IN) :: arch(*)
+          END SUBROUTINE
+
           ! Non-pure function returning the current clock tick
           ! using a platform-specific resolution.
           INTEGER(C_LONG_LONG) FUNCTION libxsmm_timer_tick() BIND(C)
@@ -296,20 +318,23 @@
       END INTERFACE$MNK_INTERFACE_LIST
 
       CONTAINS
-        ! Returns a name for the target architecture as identified
-        ! by libxsmm_get_target_arch().
-        !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_get_target_archid
-        PURE FUNCTION libxsmm_get_target_archid() RESULT(name)
-          CHARACTER(LEN=3) :: name
-          !DIR$ ATTRIBUTES OFFLOAD:MIC :: get_target_archid
+        ! Returns the name of the target architecture as determined by
+        ! the CPUID flags, as set by the libxsmm_get_target_arch* functions,
+        ! or as set by the LIBXSMM_TARGET environment variable.
+        !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_get_target_arch
+        FUNCTION libxsmm_get_target_arch() RESULT(arch)
+          CHARACTER(LEN=:), ALLOCATABLE :: arch
+          CHARACTER(LEN=16) :: tmp
+          !DIR$ ATTRIBUTES OFFLOAD:MIC :: get_target_arch
           INTERFACE
-            PURE SUBROUTINE get_target_archid(name, length) BIND(C)
+            PURE SUBROUTINE get_target_arch(arch, length) BIND(C)
               IMPORT :: C_CHAR, C_INT
-              CHARACTER(C_CHAR), INTENT(OUT) :: name(*)
+              CHARACTER(C_CHAR), INTENT(OUT) :: arch(*)
               INTEGER(C_INT), VALUE, INTENT(IN) :: length
             END SUBROUTINE
           END INTERFACE
-          CALL get_target_archid(name, LEN(name))
+          CALL get_target_arch(tmp, LEN(tmp))
+          arch = TRIM(tmp)
         END FUNCTION
 
         !DIR$ ATTRIBUTES OFFLOAD:MIC :: srealptr
@@ -338,6 +363,7 @@
           !DIR$ ATTRIBUTES OFFLOAD:MIC :: fn1
           PROCEDURE(LIBXSMM_MMFUNCTION1), POINTER :: fn1
           !DIR$ ATTRIBUTES OFFLOAD:MIC :: sdispatch
+          INTEGER(C_INT) :: oprefetch
           INTERFACE
             TYPE(C_FUNPTR) PURE FUNCTION sdispatch(                     &
      &      m, n, k, lda, ldb, ldc, alpha, beta, flags, prefetch)       &
@@ -349,9 +375,12 @@
               INTEGER(C_INT), INTENT(IN) :: flags, prefetch
             END FUNCTION
           END INTERFACE
-          IF (.NOT.PRESENT(prefetch).OR.                                &
-     &        LIBXSMM_PREFETCH_NONE.EQ.prefetch)                        &
-     &    THEN
+          IF (.NOT.PRESENT(prefetch)) THEN
+            oprefetch = LIBXSMM_PREFETCH_NONE
+          ELSE
+            oprefetch = prefetch
+          END IF
+          IF (LIBXSMM_PREFETCH_NONE.EQ.oprefetch) THEN
             CALL C_F_PROCPOINTER(sdispatch(m, n, k,                     &
      &        lda, ldb, ldc, alpha, beta, flags, prefetch),             &
      &        fn0)
@@ -378,6 +407,7 @@
           !DIR$ ATTRIBUTES OFFLOAD:MIC :: fn1
           PROCEDURE(LIBXSMM_MMFUNCTION1), POINTER :: fn1
           !DIR$ ATTRIBUTES OFFLOAD:MIC :: ddispatch
+          INTEGER(C_INT) :: oprefetch
           INTERFACE
             PURE FUNCTION ddispatch(                                    &
      &      m, n, k, lda, ldb, ldc, alpha, beta, flags, prefetch)       &
@@ -390,9 +420,12 @@
               TYPE(C_FUNPTR) :: fn
             END FUNCTION
           END INTERFACE
-          IF (.NOT.PRESENT(prefetch).OR.                                &
-     &        LIBXSMM_PREFETCH_NONE.EQ.prefetch)                        &
-     &    THEN
+          IF (.NOT.PRESENT(prefetch)) THEN
+            oprefetch = LIBXSMM_PREFETCH_NONE
+          ELSE
+            oprefetch = prefetch
+          END IF
+          IF (LIBXSMM_PREFETCH_NONE.EQ.oprefetch) THEN
             CALL C_F_PROCPOINTER(ddispatch(m, n, k,                     &
      &        lda, ldb, ldc, alpha, beta, flags, prefetch),             &
      &        fn0)
@@ -710,4 +743,121 @@
           CALL internal_gemm(transa, transb, m, n, k,                   &
      &      alpha, a, lda, b, ldb, beta, c, ldc)
         END SUBROUTINE
+
+        !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_smatmul
+        SUBROUTINE libxsmm_smatmul(c, a, b, alpha, beta, transa, transb)
+          REAL(C_FLOAT), INTENT(INOUT) :: c(:,:)
+          REAL(C_FLOAT), INTENT(IN) :: a(:,:), b(:,:)
+          REAL(C_FLOAT), INTENT(IN), OPTIONAL :: alpha, beta
+          CHARACTER, INTENT(IN), OPTIONAL :: transa, transb
+          CHARACTER :: otransa, otransb
+          IF (.NOT.PRESENT(transa)) THEN
+            otransa = 'N'
+          ELSE
+            otransa = transa
+          END IF
+          IF (.NOT.PRESENT(transb)) THEN
+            otransb = 'N'
+          ELSE
+            otransb = transb
+          END IF
+          ! TODO: transpose is currently not supported by LIBXSMM
+          IF (('N'.EQ.otransa).OR.('n'.EQ.otransa)) THEN
+            IF (('N'.EQ.otransb).OR.('n'.EQ.otransb)) THEN
+              CALL libxsmm_sgemm('N', 'N',                              &
+     &          SIZE(c, 1, LIBXSMM_BLASINT_KIND),                       &
+     &          SIZE(c, 2, LIBXSMM_BLASINT_KIND),                       &
+     &          SIZE(a, 2, LIBXSMM_BLASINT_KIND),                       &
+     &          alpha, a, SIZE(a, 1, LIBXSMM_BLASINT_KIND),             &
+     &                 b, SIZE(b, 1, LIBXSMM_BLASINT_KIND),             &
+     &           beta, c, SIZE(c, 1, LIBXSMM_BLASINT_KIND))
+            ELSE ! A x B^T
+              CALL libxsmm_sgemm('N', 'N',                              &
+     &          SIZE(c, 1, LIBXSMM_BLASINT_KIND),                       &
+     &          SIZE(c, 2, LIBXSMM_BLASINT_KIND),                       &
+     &          SIZE(a, 2, LIBXSMM_BLASINT_KIND),                       &
+     &          alpha, a, SIZE(a, 1, LIBXSMM_BLASINT_KIND),             &
+     &          TRANSPOSE(b), SIZE(b, 2, LIBXSMM_BLASINT_KIND),         &
+     &          beta, c, SIZE(c, 1, LIBXSMM_BLASINT_KIND))
+            END IF
+          ELSE ! A^T
+            IF (('N'.EQ.otransb).OR.('n'.EQ.otransb)) THEN
+              CALL libxsmm_sgemm('N', 'N',                              &
+     &          SIZE(c, 1, LIBXSMM_BLASINT_KIND),                       &
+     &          SIZE(c, 2, LIBXSMM_BLASINT_KIND),                       &
+     &          SIZE(a, 1, LIBXSMM_BLASINT_KIND),                       &
+     &          alpha, TRANSPOSE(a), SIZE(a, 2, LIBXSMM_BLASINT_KIND),  &
+     &                 b, SIZE(b, 1, LIBXSMM_BLASINT_KIND),             &
+     &           beta, c, SIZE(c, 1, LIBXSMM_BLASINT_KIND))
+            ELSE ! A^T x B^T -> C = (B x A)^T
+              CALL libxsmm_sgemm('N', 'N',                              &
+     &          SIZE(c, 1, LIBXSMM_BLASINT_KIND),                       &
+     &          SIZE(c, 2, LIBXSMM_BLASINT_KIND),                       &
+     &          SIZE(a, 1, LIBXSMM_BLASINT_KIND),                       &
+     &          alpha, b, SIZE(b, 1, LIBXSMM_BLASINT_KIND),             &
+     &                 a, SIZE(a, 1, LIBXSMM_BLASINT_KIND),             &
+     &           beta, c, SIZE(c, 1, LIBXSMM_BLASINT_KIND))
+              c = TRANSPOSE(RESHAPE(c, (/ SIZE(c, 2), SIZE(c, 1) /)))
+            END IF
+          END IF
+        END SUBROUTINE
+
+        !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_dmatmul
+        SUBROUTINE libxsmm_dmatmul(c, a, b, alpha, beta, transa, transb)
+          REAL(C_DOUBLE), INTENT(INOUT) :: c(:,:)
+          REAL(C_DOUBLE), INTENT(IN) :: a(:,:), b(:,:)
+          REAL(C_DOUBLE), INTENT(IN), OPTIONAL :: alpha, beta
+          CHARACTER, INTENT(IN), OPTIONAL :: transa, transb
+          CHARACTER :: otransa, otransb
+          IF (.NOT.PRESENT(transa)) THEN
+            otransa = 'N'
+          ELSE
+            otransa = transa
+          END IF
+          IF (.NOT.PRESENT(transb)) THEN
+            otransb = 'N'
+          ELSE
+            otransb = transb
+          END IF
+          ! TODO: transpose is currently not supported by LIBXSMM
+          IF (('N'.EQ.otransa).OR.('n'.EQ.otransa)) THEN
+            IF (('N'.EQ.otransb).OR.('n'.EQ.otransb)) THEN
+              CALL libxsmm_dgemm('N', 'N',                              &
+     &          SIZE(c, 1, LIBXSMM_BLASINT_KIND),                       &
+     &          SIZE(c, 2, LIBXSMM_BLASINT_KIND),                       &
+     &          SIZE(a, 2, LIBXSMM_BLASINT_KIND),                       &
+     &          alpha, a, SIZE(a, 1, LIBXSMM_BLASINT_KIND),             &
+     &                 b, SIZE(b, 1, LIBXSMM_BLASINT_KIND),             &
+     &           beta, c, SIZE(c, 1, LIBXSMM_BLASINT_KIND))
+            ELSE ! A x B^T
+              CALL libxsmm_dgemm('N', 'N',                              &
+     &          SIZE(c, 1, LIBXSMM_BLASINT_KIND),                       &
+     &          SIZE(c, 2, LIBXSMM_BLASINT_KIND),                       &
+     &          SIZE(a, 2, LIBXSMM_BLASINT_KIND),                       &
+     &          alpha, a, SIZE(a, 1, LIBXSMM_BLASINT_KIND),             &
+     &          TRANSPOSE(b), SIZE(b, 2, LIBXSMM_BLASINT_KIND),         &
+     &          beta, c, SIZE(c, 1, LIBXSMM_BLASINT_KIND))
+            END IF
+          ELSE ! A^T
+            IF (('N'.EQ.otransb).OR.('n'.EQ.otransb)) THEN
+              CALL libxsmm_dgemm('N', 'N',                              &
+     &          SIZE(c, 1, LIBXSMM_BLASINT_KIND),                       &
+     &          SIZE(c, 2, LIBXSMM_BLASINT_KIND),                       &
+     &          SIZE(a, 1, LIBXSMM_BLASINT_KIND),                       &
+     &          alpha, TRANSPOSE(a), SIZE(a, 2, LIBXSMM_BLASINT_KIND),  &
+     &                 b, SIZE(b, 1, LIBXSMM_BLASINT_KIND),             &
+     &           beta, c, SIZE(c, 1, LIBXSMM_BLASINT_KIND))
+            ELSE ! A^T x B^T -> C = (B x A)^T
+              CALL libxsmm_dgemm('N', 'N',                              &
+     &          SIZE(c, 1, LIBXSMM_BLASINT_KIND),                       &
+     &          SIZE(c, 2, LIBXSMM_BLASINT_KIND),                       &
+     &          SIZE(a, 1, LIBXSMM_BLASINT_KIND),                       &
+     &          alpha, b, SIZE(b, 1, LIBXSMM_BLASINT_KIND),             &
+     &                 a, SIZE(a, 1, LIBXSMM_BLASINT_KIND),             &
+     &           beta, c, SIZE(c, 1, LIBXSMM_BLASINT_KIND))
+              c = TRANSPOSE(RESHAPE(c, (/ SIZE(c, 2), SIZE(c, 1) /)))
+            END IF
+          END IF
+        END SUBROUTINE
       END MODULE
+
