@@ -44,20 +44,14 @@
 # pragma offload_attribute(pop)
 #endif
 
-#if defined(LIBXSMM_MAX_STATIC_TARGET_ARCH) && (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE /*|| any other implemented size*/)
-# if (LIBXSMM_X86_AVX512_CORE <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
-#   define LIBXSMM_GEMM_DIFF_AVX512_CORE
-# elif (LIBXSMM_X86_AVX512_MIC <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
-#   define LIBXSMM_GEMM_DIFF_AVX512_MIC
-# elif (LIBXSMM_X86_AVX2 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
-#   define LIBXSMM_GEMM_DIFF_AVX2
-# elif (LIBXSMM_X86_AVX <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
-#   define LIBXSMM_GEMM_DIFF_AVX
-# else
-#   define LIBXSMM_GEMM_DIFF_NOWARNING
-# endif
-#else /* remove as soon as LIBXSMM_GEMM_DESCRIPTOR_SIZE=16 is supported */
-# define LIBXSMM_GEMM_DIFF_NOWARNING
+/* individually enable intrinsic code paths */
+#if defined(LIBXSMM_MAX_STATIC_TARGET_ARCH)
+# define LIBXSMM_GEMM_DIFF_AVX512_CORE
+# define LIBXSMM_GEMM_DIFF_AVX512_MIC
+# define LIBXSMM_GEMM_DIFF_AVX2
+# define LIBXSMM_GEMM_DIFF_AVX
+# define LIBXSMM_GEMM_DIFF_SSE
+/*# define LIBXSMM_GEMM_DIFF_KNC*/
 #endif
 
 
@@ -88,12 +82,17 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void libxsmm_gemm_diff_init(int target_arc
     internal_gemm_diffn_function = libxsmm_gemm_diffn_avx;
     internal_gemm_diff_function = libxsmm_gemm_diff_avx;
   }
+  else if (LIBXSMM_X86_SSE3 <= target_arch) {
+    internal_gemm_diff_function = libxsmm_gemm_diff_sse;
+  }
 #endif
 }
 
 
 LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void libxsmm_gemm_diff_finalize(void)
 {
+  internal_gemm_diff_function = libxsmm_gemm_diff_sw;
+  internal_gemm_diffn_function = libxsmm_gemm_diffn_sw;
 }
 
 
@@ -109,6 +108,8 @@ unsigned int libxsmm_gemm_diff(const libxsmm_gemm_descriptor* reference, const l
   return libxsmm_gemm_diff_avx2(reference, desc);
 #elif defined(LIBXSMM_STATIC_TARGET_ARCH) && (LIBXSMM_X86_AVX <= LIBXSMM_STATIC_TARGET_ARCH)
   return libxsmm_gemm_diff_avx(reference, desc);
+#elif defined(LIBXSMM_STATIC_TARGET_ARCH) && (LIBXSMM_X86_SSE3 <= LIBXSMM_STATIC_TARGET_ARCH)
+  return libxsmm_gemm_diff_sse(reference, desc);
 #else /* pointer based function call */
   assert(0 != internal_gemm_diff_function);
   return (*internal_gemm_diff_function)(reference, desc);
@@ -136,15 +137,46 @@ unsigned int libxsmm_gemm_diff_sw(const libxsmm_gemm_descriptor* reference, cons
 }
 
 
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS
+unsigned int libxsmm_gemm_diff_sse(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* desc)
+{
+  assert(0 != reference && 0 != desc);
+#if defined(LIBXSMM_GEMM_DIFF_SSE) && (LIBXSMM_X86_SSE3 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+  assert(0 == LIBXSMM_MOD2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
+  assert(4 >= LIBXSMM_DIV2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
+# if (16 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
+  {
+    const __m128i a128 = _mm_lddqu_si128((const __m128i*)reference);
+    const __m128i b128 = _mm_lddqu_si128((const __m128i*)desc);
+    const __m128i c128 = _mm_cmpeq_epi8(a128, b128);
+    return 0xFFFF != _mm_movemask_epi8(c128);
+  }
+# else
+  return libxsmm_gemm_diff_sw(reference, desc);
+# endif
+#else
+# if !defined(NDEBUG) && defined(LIBXSMM_GEMM_DIFF_SSE) /* library code is expected to be mute */
+  { static LIBXSMM_TLS int once = 0;
+    if (0 == once) {
+      fprintf(stderr, "LIBXSMM: unable to enter SSE code path!\n");
+      once = 1;
+    }
+  }
+# endif
+  return libxsmm_gemm_diff_sw(reference, desc);
+#endif
+}
+
+
 LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS
 unsigned int libxsmm_gemm_diff_avx(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* desc)
 {
-#if defined(LIBXSMM_GEMM_DIFF_AVX)
+  assert(0 != reference && 0 != desc);
+#if defined(LIBXSMM_GEMM_DIFF_AVX) && (LIBXSMM_X86_AVX <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
   assert(0 == LIBXSMM_MOD2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
   assert(8 >= LIBXSMM_DIV2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
-  assert(0 != reference && 0 != desc);
-  {
 # if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
+  {
     int r0, r1;
     __m256i a256, b256;
 #   if defined(__CYGWIN__) && !defined(NDEBUG) /* Cygwin/GCC: _mm256_set_epi32 may cause an illegal instruction */
@@ -164,20 +196,26 @@ unsigned int libxsmm_gemm_diff_avx(const libxsmm_gemm_descriptor* reference, con
     a256 = _mm256_loadu_si256((const __m256i*)reference);
 #   endif
     b256 = _mm256_castps_si256(_mm256_maskload_ps((const float*)desc, m256.i));
+    /* avoid warning about eval. in unspecified order: r0, r1 */
     r0 = _mm256_testnzc_si256(a256, b256);
     r1 = _mm256_testnzc_si256(b256, a256);
     return r0 | r1;
-# endif
   }
+# else
+  { /* no difference between SSE and AVX based implementation */
+    return libxsmm_gemm_diff_sse(reference, desc);
+  }
+# endif
 #else
-# if !defined(NDEBUG) && !defined(LIBXSMM_GEMM_DIFF_NOWARNING) /* library code is expected to be mute */
-  static LIBXSMM_TLS int once = 0;
-  if (0 == once) {
-    fprintf(stderr, "LIBXSMM: unable to enter AVX code path!\n");
-    once = 1;
+# if !defined(NDEBUG) && defined(LIBXSMM_GEMM_DIFF_AVX) /* library code is expected to be mute */
+  { static LIBXSMM_TLS int once = 0;
+    if (0 == once) {
+      fprintf(stderr, "LIBXSMM: unable to enter AVX code path!\n");
+      once = 1;
+    }
   }
 # endif
-  return libxsmm_gemm_diff_sw(reference, desc);
+  return libxsmm_gemm_diff_sse(reference, desc);
 #endif
 }
 
@@ -185,12 +223,12 @@ unsigned int libxsmm_gemm_diff_avx(const libxsmm_gemm_descriptor* reference, con
 LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS
 unsigned int libxsmm_gemm_diff_avx2(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* desc)
 {
-#if defined(LIBXSMM_GEMM_DIFF_AVX2)
+  assert(0 != reference && 0 != desc);
+#if defined(LIBXSMM_GEMM_DIFF_AVX2) && (LIBXSMM_X86_AVX2 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
   assert(0 == LIBXSMM_MOD2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
   assert(8 >= LIBXSMM_DIV2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
-  assert(0 != reference && 0 != desc);
-  {
 # if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
+  {
     const int yes = 0x80000000, no = 0x0;
     const __m256i m256 = _mm256_set_epi32(no, yes, yes, yes, yes, yes, yes, yes);
 #   if defined(LIBXSMM_GEMM_DIFF_MASK_A) || !defined(LIBXSMM_GEMM_DIFF_ZERO_PADDED)
@@ -200,17 +238,25 @@ unsigned int libxsmm_gemm_diff_avx2(const libxsmm_gemm_descriptor* reference, co
     const __m256i a256 = _mm256_loadu_si256((const __m256i*)reference);
 #   endif
     const __m256i b256 = _mm256_maskload_epi32((const void*)desc, m256);
+    /* avoid warning about eval. in unspecified order: r0, r1 */
     const int r0 = _mm256_testnzc_si256(a256, b256);
     const int r1 = _mm256_testnzc_si256(b256, a256);
     return r0 | r1;
-# endif
   }
+# elif (16 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
+  { /* no difference between AVX and AVX2 based implementation */
+    return libxsmm_gemm_diff_avx(reference, desc);
+  }
+# else
+  return libxsmm_gemm_diff_avx(reference, desc);
+# endif
 #else
-# if !defined(NDEBUG) && !defined(LIBXSMM_GEMM_DIFF_NOWARNING) /* library code is expected to be mute */
-  static LIBXSMM_TLS int once = 0;
-  if (0 == once) {
-    fprintf(stderr, "LIBXSMM: unable to enter AVX2 code path!\n");
-    once = 1;
+# if !defined(NDEBUG) && defined(LIBXSMM_GEMM_DIFF_AVX2) /* library code is expected to be mute */
+  { static LIBXSMM_TLS int once = 0;
+    if (0 == once) {
+      fprintf(stderr, "LIBXSMM: unable to enter AVX2 code path!\n");
+      once = 1;
+    }
   }
 # endif
   return libxsmm_gemm_diff_avx(reference, desc);
@@ -221,10 +267,10 @@ unsigned int libxsmm_gemm_diff_avx2(const libxsmm_gemm_descriptor* reference, co
 LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE
 unsigned int libxsmm_gemm_diff_imci(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* desc)
 {
-#if defined(__MIC__) && (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE/*|| any other implemented size*/)
+  assert(0 != reference && 0 != desc);
+#if defined(__MIC__) && (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
   assert(0 ==  LIBXSMM_MOD2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
   assert(16 >= LIBXSMM_DIV2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
-  assert(0 != reference && 0 != desc);
   {
     const __mmask16 mask = (0xFFFF >> (16 - (LIBXSMM_GEMM_DESCRIPTOR_SIZE >> 2/*LOG2(sizeof(int))*/)));
     const __m512i a512 = _mm512_mask_loadunpackhi_epi32(
@@ -274,7 +320,7 @@ unsigned int libxsmm_gemm_diffn_sw(const libxsmm_gemm_descriptor* reference, con
   for (i = hint; i < end; ++i) {
     const unsigned int j = i % ndescs; /* wrap around index */
     /* negative stride runs backwards */
-    if (0 == libxsmm_gemm_diff(reference, (const libxsmm_gemm_descriptor*)(desc + j * nbytes))) {
+    if (0 == libxsmm_gemm_diff_sw(reference, (const libxsmm_gemm_descriptor*)(desc + j * nbytes))) {
       return j;
     }
   }
@@ -282,7 +328,7 @@ unsigned int libxsmm_gemm_diffn_sw(const libxsmm_gemm_descriptor* reference, con
   assert(/*is pot*/ndescs == (1u << LIBXSMM_LOG2(ndescs)));
   for (i = hint; i < end; ++i) {
     const unsigned int j = LIBXSMM_MOD2(i, ndescs); /* wrap around index */
-    if (0 == libxsmm_gemm_diff(reference, (const libxsmm_gemm_descriptor*)(desc + j * nbytes))) {
+    if (0 == libxsmm_gemm_diff_sw(reference, (const libxsmm_gemm_descriptor*)(desc + j * nbytes))) {
       return j;
     }
   }
@@ -295,11 +341,11 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS
 unsigned int libxsmm_gemm_diffn_avx(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
   unsigned int hint, unsigned int ndescs, int nbytes)
 {
-#if defined(LIBXSMM_GEMM_DIFF_AVX)
+#if defined(LIBXSMM_GEMM_DIFF_AVX) && (LIBXSMM_X86_AVX <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
   assert(/*is pot*/ndescs == (1u << LIBXSMM_LOG2(ndescs)));
+# if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
   assert(32 == nbytes); /* padded descriptor array */
   {
-# if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
     const unsigned int end = hint + ndescs;
     const char *const desc = (const char*)descs;
     __m256i a256;
@@ -331,15 +377,23 @@ unsigned int libxsmm_gemm_diffn_avx(const libxsmm_gemm_descriptor* reference, co
         return j;
       }
     }
-# endif
+    return ndescs;
   }
-  return ndescs;
+# elif (16 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
+  assert(16 == nbytes); /* padded descriptor array */
+  { /* TODO: implement for 16 Byte descriptor */
+    return libxsmm_gemm_diffn_sw(reference, descs, hint, ndescs, nbytes);
+  }
+# else
+  return libxsmm_gemm_diffn_sw(reference, descs, hint, ndescs, nbytes);
+# endif
 #else
-# if !defined(NDEBUG) && !defined(LIBXSMM_GEMM_DIFF_NOWARNING) /* library code is expected to be mute */
-  static LIBXSMM_TLS int once = 0;
-  if (0 == once) {
-    fprintf(stderr, "LIBXSMM: unable to enter AVX code path!\n");
-    once = 1;
+# if !defined(NDEBUG) && defined(LIBXSMM_GEMM_DIFF_AVX) /* library code is expected to be mute */
+  { static LIBXSMM_TLS int once = 0;
+    if (0 == once) {
+      fprintf(stderr, "LIBXSMM: unable to enter AVX code path!\n");
+      once = 1;
+    }
   }
 # endif
   return libxsmm_gemm_diffn_sw(reference, descs, hint, ndescs, nbytes);
@@ -351,11 +405,11 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS
 unsigned int libxsmm_gemm_diffn_avx2(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
   unsigned int hint, unsigned int ndescs, int nbytes)
 {
-#if defined(LIBXSMM_GEMM_DIFF_AVX2)
+#if defined(LIBXSMM_GEMM_DIFF_AVX2) && (LIBXSMM_X86_AVX2 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
   assert(/*is pot*/ndescs == (1u << LIBXSMM_LOG2(ndescs)));
+# if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
   assert(32 == nbytes); /* padded descriptor array */
   {
-# if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
     const unsigned int end = hint + ndescs;
     const char *const desc = (const char*)descs;
     unsigned int i;
@@ -378,15 +432,23 @@ unsigned int libxsmm_gemm_diffn_avx2(const libxsmm_gemm_descriptor* reference, c
         return j;
       }
     }
-# endif
+    return ndescs;
   }
-  return ndescs;
+# elif (16 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
+  assert(16 == nbytes); /* padded descriptor array */
+  { /* TODO: implement for 16 Byte descriptor */
+    return libxsmm_gemm_diffn_avx(reference, descs, hint, ndescs, nbytes);
+  }
+# else
+  return libxsmm_gemm_diffn_avx(reference, descs, hint, ndescs, nbytes);
+# endif
 #else
-# if !defined(NDEBUG) && !defined(LIBXSMM_GEMM_DIFF_NOWARNING) /* library code is expected to be mute */
-  static LIBXSMM_TLS int once = 0;
-  if (0 == once) {
-    fprintf(stderr, "LIBXSMM: unable to enter AVX2 code path!\n");
-    once = 1;
+# if !defined(NDEBUG) && defined(LIBXSMM_GEMM_DIFF_AVX2) /* library code is expected to be mute */
+  { static LIBXSMM_TLS int once = 0;
+    if (0 == once) {
+      fprintf(stderr, "LIBXSMM: unable to enter AVX2 code path!\n");
+      once = 1;
+    }
   }
 # endif
   return libxsmm_gemm_diffn_avx(reference, descs, hint, ndescs, nbytes);
@@ -398,11 +460,12 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS
 unsigned int libxsmm_gemm_diffn_avx512(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
   unsigned int hint, unsigned int ndescs, int nbytes)
 {
-#if defined(LIBXSMM_GEMM_DIFF_AVX512)
+#if (defined(LIBXSMM_GEMM_DIFF_AVX512_MIC)  && LIBXSMM_X86_AVX512_MIC  <= LIBXSMM_MAX_STATIC_TARGET_ARCH) || \
+    (defined(LIBXSMM_GEMM_DIFF_AVX512_CORE) && LIBXSMM_X86_AVX512_CORE <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
   assert(/*is pot*/ndescs == (1 << LIBXSMM_LOG2(ndescs)));
+# if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
   assert(32 == nbytes); /* padded descriptor array */
   {
-# if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
     const unsigned int hint_even = (hint & 0xFFFFFFFE), end = hint_even + ndescs;
     const char *const desc = (const char*)descs;
     unsigned int i;
@@ -427,15 +490,23 @@ unsigned int libxsmm_gemm_diffn_avx512(const libxsmm_gemm_descriptor* reference,
         return j + 1;
       }
     }
-# endif
+    return ndescs;
   }
-  return ndescs;
+# elif (16 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
+  assert(16 == nbytes); /* padded descriptor array */
+  { /* TODO: implement for 16 Byte descriptor */
+    return libxsmm_gemm_diffn_avx2(reference, descs, hint, ndescs, nbytes);
+  }
+# else
+  return libxsmm_gemm_diffn_avx2(reference, descs, hint, ndescs, nbytes);
+# endif
 #else
-# if !defined(NDEBUG) && !defined(LIBXSMM_GEMM_DIFF_NOWARNING) /* library code is expected to be mute */
-  static LIBXSMM_TLS int once = 0;
-  if (0 == once) {
-    fprintf(stderr, "LIBXSMM: unable to enter AVX-512 code path!\n");
-    once = 1;
+# if !defined(NDEBUG) && (defined(LIBXSMM_GEMM_DIFF_AVX512_MIC) || defined(LIBXSMM_GEMM_DIFF_AVX512_CORE))
+  { static LIBXSMM_TLS int once = 0;
+    if (0 == once) {
+      fprintf(stderr, "LIBXSMM: unable to enter AVX-512 code path!\n");
+      once = 1;
+    }
   }
 # endif
   return libxsmm_gemm_diffn_avx2(reference, descs, hint, ndescs, nbytes);
@@ -447,7 +518,7 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE
 unsigned int libxsmm_gemm_diffn_imci(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
   unsigned int hint, unsigned int ndescs, int nbytes)
 {
-#if defined(__MIC__) && (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE/*|| any other implemented size*/) && 0/*disabled*/
+#if defined(LIBXSMM_GEMM_DIFF_KNC) && defined(__MIC__) && (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
   assert(/*is pot*/ndescs == (1 << LIBXSMM_LOG2(ndescs)));
   assert(32 == nbytes); /* padded descriptor array */
   {
@@ -471,8 +542,14 @@ unsigned int libxsmm_gemm_diffn_imci(const libxsmm_gemm_descriptor* reference, c
         return j + 1;
       }
     }
+    return ndescs;
+  }
+#elif defined(LIBXSMM_GEMM_DIFF_KNC) && defined(__MIC__) && (16 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
+  assert(/*is pot*/ndescs == (1 << LIBXSMM_LOG2(ndescs)));
+  assert(16 == nbytes); /* padded descriptor array */
+  { /* TODO: implement for 16 Byte descriptor */
+    return libxsmm_gemm_diffn_sw(reference, descs, hint, ndescs, nbytes);
   }
-  return ndescs;
 #else
   return libxsmm_gemm_diffn_sw(reference, descs, hint, ndescs, nbytes);
 #endif
