@@ -36,8 +36,14 @@
 #if defined(__MIC__)
 # define LIBXSMM_STATIC_TARGET_ARCH LIBXSMM_X86_IMCI
 #else
-# if defined(__AVX512F__)
+# if  defined(__AVX512F__)  && defined(__AVX512CD__) \
+   && defined(__AVX512DQ__) && defined(__AVX512BW__) && defined(__AVX512VL__)
 #   define LIBXSMM_STATIC_TARGET_ARCH LIBXSMM_X86_AVX512_CORE
+# elif defined(__AVX512F__) && defined(__AVX512CD__) \
+   && defined(__AVX512PF__) && defined(__AVX512ER__)
+#   define LIBXSMM_STATIC_TARGET_ARCH LIBXSMM_X86_AVX512_MIC
+# elif defined(__AVX512F__)
+    /* TODO: adjust this defintion if we go beyond AVX-512F */
 #   define LIBXSMM_STATIC_TARGET_ARCH LIBXSMM_X86_AVX512_MIC
 # elif defined(__AVX2__)
 #   define LIBXSMM_STATIC_TARGET_ARCH LIBXSMM_X86_AVX2
